unit GWCOMDemo_COMObjectsOut;

interface
uses GWCOMDemo_Base;

Type
  //-------------------------------
  TGWCOMDemo_COMObjectsOut = class(TGWCOMDemo_Base)
  //-------------------------------
  public
    procedure GetObject_Proc(out COM_Interface_A: IUnknown);
    function  GetObject_Func: IUnknown;
    function  GetObject_Func_OneStep: IUnknown;
    procedure Demo_Run_Inner;
    procedure Demo_Run; override;
  end;

//-------------------------------
implementation
//-------------------------------
uses GWMonOLE, GWDemoCode;

//ShowBegin
// Demonstrate returning COM objects from procedures and functions

//---------------------------------------
procedure TGWCOMDemo_COMObjectsOut.GetObject_Proc(out COM_Interface_A: IUnknown);
//---------------------------------------
Var
  Pascal_Object_A: TGWMonInterfacedObject;
Begin
  Pascal_Object_A := TGWMonInterfacedObject.Create(Mon);
  Pause('A-010');  // object created, but RefCount still zero
  COM_Interface_A := Pascal_Object_A;
  Pause('A-020'); // We made a reference and have a RefCount of 1
end;

//---------------------------------------
function TGWCOMDemo_COMObjectsOut.GetObject_Func: IUnknown;
//---------------------------------------
Var
  Pascal_Object_A: TGWMonInterfacedObject;
Begin
  Pascal_Object_A := TGWMonInterfacedObject.Create(Mon);
  Pause('B-010'); // object created, but RefCount still zero

  Result := Pascal_Object_A;
  Pause('B-020'); // We made a reference and have a RefCount of 1
end;

//---------------------------------------
function TGWCOMDemo_COMObjectsOut.GetObject_Func_OneStep: IUnknown;
//---------------------------------------
Begin  // same as GetObject_Func, but just one step
  Result := TGWMonInterfacedObject.Create(Mon);
  Pause('C-020'); // We made a reference and have a RefCount of 1
end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsOut.Demo_Run_Inner;
//---------------------------------------
Var
  COM_Interface_1:  IUnknown;
Begin
  Pause('000');

  //-----------------------------------------------
  // Get COM object through "out" parameter
  //-----------------------------------------------
  GetObject_Proc(COM_Interface_1);
  Pause('010');  // RefCount of received object is 1

  COM_Interface_1 := nil;
  Pause('020'); // We lost the reference, RefCount decremented, object destroys

  //-----------------------------------------------
  // Get COM object through function return
  //-----------------------------------------------
  COM_Interface_1 := GetObject_Func;
  Pause('030');

  COM_Interface_1 := nil;  // destroy it
  Pause('040');

  //-----------------------------------------------
  // Again get COM object through function return,
  // ... function is simplified, same result
  //-----------------------------------------------
  COM_Interface_1 := GetObject_Func_OneStep;
  Pause('050');

  COM_Interface_1 := nil;  // destroy it
  Pause('060');

  //-----------------------------------------------
  // What if we don't assign function return value?
  //-----------------------------------------------
  GetObject_Func;
  Pause('950');   // ... it's not destroyed, but will be when we leave this proc
end;

//---------------------------------------
procedure TGWCOMDemo_COMObjectsOut.Demo_Run;
//---------------------------------------
Begin
  Pause('Begin');
  Demo_Run_Inner;
  Pause('End');   // Check that all objects are deleted by now!
end;
//ShowEnd

initialization
  DemoCode_Mgr.Add(TGWCOMDemo_COMObjectsOut.Create('COM Demo -- Return COM Objects FROM procs/funcs','GWCOMDemo_COMObjectsOut.pas'));
end.

